@@ -13,7 +13,7 @@ def normalize_identifiers(expression: E, dialect: DialectType = None) -> E:
 
 
 @t.overload
-def normalize_identifiers(expression: str, dialect: DialectType = None) -> exp.Expression:
+def normalize_identifiers(expression: str, dialect: DialectType = None) -> exp.Identifier:
     ...
 
 
@@ -48,11 +48,11 @@ def normalize_identifiers(expression, dialect=None):
     Returns:
         The transformed expression.
     """
+    dialect = Dialect.get_or_raise(dialect)
+
     if isinstance(expression, str):
         expression = exp.parse_identifier(expression, dialect=dialect)
 
-    dialect = Dialect.get_or_raise(dialect)
-
     def _normalize(node: E) -> E:
         if not node.meta.get("case_sensitive"):
             exp.replace_children(node, _normalize)
