@@ -865,5 +865,8 @@ KILL CONNECTION 123
 KILL QUERY '123'
 CHR(97)
 SELECT * FROM UNNEST(x) WITH ORDINALITY UNION ALL SELECT * FROM UNNEST(y) WITH ORDINALITY
+SELECT x FROM t1 UNION ALL SELECT x FROM t2 LIMIT 1
+SELECT x FROM t1 UNION ALL SELECT x FROM t2 UNION ALL SELECT x FROM t3 LIMIT 1
 WITH use(use) AS (SELECT 1) SELECT use FROM use
 SELECT recursive FROM t
+SELECT (ROW_NUMBER() OVER (PARTITION BY user ORDER BY date ASC) - ROW_NUMBER() OVER (PARTITION BY user, segment ORDER BY date ASC)) AS group_id FROM example_table
